#include<stdio.h>
#include<math.h>
#include <unistd.h>
#include <stdlib.h>

#include<libnova/ln_types.h>
#include<libnova/lunar.h>
#include<libnova/solar.h>
#include<libnova/julian_day.h>

#define PRINT_HORA(DJ,FLAG) {\
ln_get_date (DJ, &date);\
if( (DJ>=(JD_tmp+1.)) || (DJ<(JD_tmp-1.)) || FLAG )printf("| **  **  ");\
else {\
if(date.seconds>30)date.minutes++;\
if(date.minutes>=60){date.hours++;date.minutes-=60;}\
printf("| %2dh %02dm ", date.hours, date.minutes);\
}\
}

#define LINEA_HORIZONTAL "+------------+---------+---------+---------+---------+---------+---------+-----+\n"

#define LINEA_INFO "Info: monje314-2006@yahoo.es - Powred by libnova: http://libnova.sourceforge.net"

void usage(char *exe){
printf("usage.....\n");
if(exe!=NULL)printf("%s <-i date_start> <-f date_stop> <-L long> <-l lat> [-O site_label]\n\n", exe);
printf("Date format --> YYYY:MM:DD\n");
printf("Lon/Lat format --> +/-GG:MM:SS\n\n");
exit(1);
}

double cal_dj(char *str){
struct ln_date date;

/*default values*/
/*date.years=2000;
date.months=1;
date.days=1;
date.hours=0;
date.minutes=0;
date.seconds=0.;*/

if(3!=sscanf(str, "%d:%d:%d", &(date.years), &(date.months), &(date.days))){
usage(NULL);
exit(1);
}

return ln_get_julian_day(&date);
}

double cal_sesag(char *str){
int tmp;
float g, m, s;

tmp=sscanf(str, "%f:%f:%f", &g, &m, &s);
if(g<0.){m=-1*m;s=-1*s;}

switch (tmp){
  case 1:
    return (double)g;
  case 2:
    return (double)(g+m/60.);
  case 3:
    return (double)(g+m/60.+s/3600.);
  default:
    usage(NULL);
} /*del switch*/
return 0.;
}

void print_lonlat(double l){
double parte_ent;

/*si es negativo, imprimimos signo menos*/
if(l<0){
    printf("-");
    l=-1*l;
    }

/* los grados */
parte_ent=floor(l);
printf("%.0fg ", parte_ent);

/* los minutos */
l=(l-parte_ent)*60.;
parte_ent=floor(l);
printf("%.0f' ", parte_ent);

/* los segundos */
l=(l-parte_ent)*60.;
printf("%.0f\"", l);

return;
}

int lunarefem(double JD_star, double JD_stop, struct ln_lnlat_posn * observer, int nlxp, char *label){
double JD_tmp;
struct ln_date date;
struct ln_rst_time rst, rst2;
int nl, flag, flag2;

nl=0;
JD_tmp=JD_star;
while(JD_tmp<=JD_stop){
/*Imprimimos cabecera de pagina*/
if(!(nl%nlxp)){
   if(nl){
         printf(LINEA_HORIZONTAL);
	 printf(LINEA_INFO);
	 printf("\f");
	 }
   printf("             Efemerides para lon= "); print_lonlat(observer->lng);
   printf(" lat= "); print_lonlat(observer->lat);
   if(label!=NULL)printf(" (%s)", label);
   printf("\n");
   printf("             +---------------------------------------+-------------------------+\n");
   printf("horas en U.T.|                  SOL                  |          LUNA           |\n");
   printf(LINEA_HORIZONTAL);
   printf("|    DIA     |ORTO AST |  ORTO   |  OCASO  |OCASO AST|  ORTO   |  OCASO  |FASE |\n");
   printf(LINEA_HORIZONTAL);
}
/*Imprimimos el dia*/
ln_get_date (JD_tmp, &date);
printf("| %4d-%2d-%2d ", date.years, date.months, date.days);

/*Calculamos orto ocaso sol*/
flag2=ln_get_solar_rst(JD_tmp, observer, &rst2);
/*Calculamos orto ocaso astronomico sol*/
flag=ln_get_solar_rst_horizon(JD_tmp, observer, LN_SOLAR_ASTRONOMICAL_HORIZON, &rst);
/*Imprimimos orto astronomico sol*/
PRINT_HORA(rst.rise,flag);
/*Imprimimos orto sol*/
PRINT_HORA(rst2.rise,flag2);
/*Imprimimos ocaso sol*/
PRINT_HORA(rst2.set,flag2);
/*Imprimimos ocaso astronomico sol*/
PRINT_HORA(rst.set,flag);

/*Calculamos orto ocaso luna*/
flag=ln_get_lunar_rst(JD_tmp, observer, &rst);
/*Imprimimos orto luna*/
PRINT_HORA(rst.rise,flag);
/*Imprimimos ocaso luna*/
PRINT_HORA(rst.set,flag);
/*Imprimimos fase*/
if(flag)printf("| *** |\n");
else    printf("| %3.0f |\n", ln_get_lunar_phase (JD_tmp));

JD_tmp+=1.;
nl++;
}/*del while */
printf(LINEA_HORIZONTAL);
printf(LINEA_INFO);

return 0;
}

int main(int nargs, char **args){
double JD_star, JD_stop;
struct ln_lnlat_posn posicion;
char *label=NULL;
int nlxp=62;
char c;
const char optstring[]="i:f:L:l:O:";

/*
 * Some init
 */
/*posicion.lng=0.;
posicion.lat=0.;
JD_star=2454101.5;
JD_stop=2454465.5;*/
posicion.lng=NAN;
posicion.lat=NAN;
JD_star=NAN;
JD_stop=NAN;

/*
 * Se procesan los argumentos
 */

while((c=getopt(nargs, args, optstring))!=-1){
  switch(c){
    case 'i':
       JD_star=cal_dj(optarg);
       break;
    case 'f':
       JD_stop=cal_dj(optarg);
       break;
    case 'L':
       posicion.lng=cal_sesag(optarg);
       break;
    case 'l':
       posicion.lat=cal_sesag(optarg);
       break;
    case 'O':
       label=optarg;
       break;
    default:
       usage(args[0]);
  }/*del switch*/
}/*del while*/

/*
According to the IEEE standard, NaN values have the odd property that comparisons involving them are always false. That is, for a float f, f != f will be true only if f is NaN. 

Note that, as some comments below have pointed out, not all compilers respect this when optimizing code.

For any compiler which claims to use IEEE floating point, this trick should work. But I can't guarantee that it will work in practice. Check with your compiler, if in doubt.

http://stackoverflow.com/questions/570669/checking-if-a-double-or-float-is-nan-in-c
*/
if((posicion.lng!=posicion.lng) || (posicion.lat!=posicion.lat) || (JD_star!=JD_star) || (JD_stop!=JD_stop)){
usage(args[0]);
exit(1);
}
/*
 * Ahora podemos llamar a la funcion principal
 */
lunarefem(JD_star, JD_stop, &posicion, nlxp, label);

return 0;
}
